@@ -53,6 +53,8 @@
 #include "um-password-dialog.h"
 #include "um-photo-dialog.h"
 #include "um-fingerprint-dialog.h"
+#include "um-utils.h"
+#include "um-strength-bar.h"
 
 #include "gdm-languages.h"
 
@@ -1118,6 +1120,7 @@ lockbutton_changed (PolkitLockButton *button,
         gboolean is_authorized;
         gboolean self_selected;
         UmUser *user;
+        GtkWidget *widget;
 
         user = get_selected_user (d);
         if (!user) {
@@ -1127,8 +1130,29 @@ lockbutton_changed (PolkitLockButton *button,
         is_authorized = polkit_lock_button_get_is_authorized (button);
         self_selected = um_user_get_uid (user) == geteuid ();
 
-        gtk_widget_set_sensitive (get_widget (d, "add-user-button"), is_authorized);
-        gtk_widget_set_sensitive (get_widget (d, "delete-user-button"), is_authorized && !self_selected);
+        widget = get_widget (d, "add-user-button");
+        gtk_widget_set_sensitive (widget, is_authorized);
+        if (is_authorized) {
+                setup_tooltip_with_embedded_icon (widget, _("Create a user"), NULL, NULL);
+        }
+        else {
+                setup_tooltip_with_embedded_icon (widget,
+                                                  _("To create a user,\nclick the * icon first"),
+                                                  "*",
+                                                  "stock_lock");
+        }
+
+        widget = get_widget (d, "delete-user-button");
+        gtk_widget_set_sensitive (widget, is_authorized && !self_selected);
+        if (is_authorized) {
+                setup_tooltip_with_embedded_icon (widget, _("Delete the selected user"), NULL, NULL);
+        }
+        else {
+                setup_tooltip_with_embedded_icon (widget,
+                                                  _("To delete the selected user,\nclick the * icon first"),
+                                                  "*",
+                                                  "stock_lock");
+        }
 
         if (is_authorized) {
                 if (gtk_notebook_get_current_page (GTK_NOTEBOOK (get_widget (d, "account-type-notebook"))) == 0) {
@@ -1186,6 +1210,69 @@ focus_moved (GtkWindow         *window,
         }
 }
 
+static gboolean
+match_user (GtkTreeModel *model,
+            gint          column,
+            const gchar  *key,
+            GtkTreeIter  *iter,
+            gpointer      search_data)
+{
+        UmUser *user;
+        const gchar *name;
+        gchar *normalized_key = NULL;
+        gchar *normalized_name = NULL;
+        gchar *case_normalized_key = NULL;
+        gchar *case_normalized_name = NULL;
+        gchar *p;
+        gboolean result = TRUE;
+        gint i;
+
+        gtk_tree_model_get (model, iter, USER_COL, &user, -1);
+
+        if (!user) {
+                goto out;
+        }
+
+        normalized_key = g_utf8_normalize (key, -1, G_NORMALIZE_ALL);
+        if (!normalized_key) {
+                goto out;
+        }
+
+        case_normalized_key = g_utf8_casefold (normalized_key, -1);
+
+        for (i = 0; i < 2; i++) {
+                if (i == 0)
+                        name = um_user_get_real_name (user);
+                else
+                        name = um_user_get_user_name (user);
+
+                g_free (normalized_name);
+                normalized_name = g_utf8_normalize (name, -1, G_NORMALIZE_ALL);
+                if (normalized_name) {
+                        g_free (case_normalized_name);
+                        case_normalized_name = g_utf8_casefold (normalized_name, -1);
+                        p = strstr (case_normalized_name, case_normalized_key);
+
+                        /* poor man's \b */
+                        if (p == case_normalized_name || (p && p[-1] == ' ')) {
+                                result = FALSE;
+                                break;
+                        }
+                }
+        }
+
+ out:
+        if (user) {
+                g_object_unref (user);
+        }
+        g_free (normalized_key);
+        g_free (case_normalized_key);
+        g_free (normalized_name);
+        g_free (case_normalized_name);
+
+        return result;
+}
+
 static void
 setup_main_window (UserAccountDialog *d)
 {
@@ -1217,6 +1304,9 @@ setup_main_window (UserAccountDialog *d)
         gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model), sort_users, NULL, NULL);
         gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
         gtk_tree_view_set_model (GTK_TREE_VIEW (userlist), model);
+        gtk_tree_view_set_search_column (GTK_TREE_VIEW (userlist), USER_COL);
+        gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (userlist),
+                                             match_user, NULL, NULL);
 
         g_signal_connect (d->um, "users-loaded", G_CALLBACK (users_loaded), d);
 
@@ -1270,17 +1360,57 @@ setup_main_window (UserAccountDialog *d)
         button = get_widget (d, "user-list-button");
         g_signal_connect (button, "toggled", G_CALLBACK (toggle_user_list), d);
 
+        button = get_widget (d, "user-icon-nonbutton");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
+        button = get_widget (d, "full-name-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "full-name-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_name), d);
 
+        button = get_widget (d, "account-type-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To change the account type,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "account-type-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_account_type_start), d);
         button = get_widget (d, "account-type-combo");
         g_signal_connect (button, "changed", G_CALLBACK (account_type_changed), d);
 
+        button = get_widget (d, "account-password-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "account-password-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_password), d);
 
+        button = get_widget (d, "account-email-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "account-email-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_email_start), d);
         button = get_widget (d, "account-email-entry");
@@ -1288,6 +1418,14 @@ setup_main_window (UserAccountDialog *d)
         g_signal_connect (button, "focus-out-event", G_CALLBACK (change_email_focus_out), d);
         g_signal_connect (button, "key-press-event", G_CALLBACK (change_email_key_press), d);
 
+        button = get_widget (d, "account-language-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "account-language-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_language_start), d);
         button = get_widget (d, "account-language-combo");
@@ -1297,6 +1435,14 @@ setup_main_window (UserAccountDialog *d)
         /* ugly hack to catch the combo boxes losing focus */
         g_signal_connect (window, "set-focus", G_CALLBACK (focus_moved), d);
 
+        button = get_widget (d, "account-location-value-label");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To make changes,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        g_signal_connect (button, "button-release-event",
+                           G_CALLBACK (show_tooltip_now), NULL);
+
         button = get_widget (d, "account-location-button");
         g_signal_connect (button, "clicked", G_CALLBACK (change_location_start), d);
         button = get_widget (d, "account-location-entry");
@@ -1316,6 +1462,17 @@ setup_main_window (UserAccountDialog *d)
                           G_CALLBACK (lockbutton_changed), d);
         lockbutton_changed (POLKIT_LOCK_BUTTON (button), d);
         d->lock_button = button;
+
+        button = get_widget (d, "add-user-button");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To create a user,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
+        button = get_widget (d, "delete-user-button");
+        setup_tooltip_with_embedded_icon (button,
+                                          _("To delete the selected user,\nclick the * icon first"),
+                                          "*",
+                                          "stock_lock");
 }
 
 static void
@@ -1347,6 +1504,7 @@ main (int argc, char *argv[])
         GError *error;
         const gchar *filename;
         GtkWidget *button;
+        volatile GType type;
         GOptionContext *context;
         static gboolean do_debug;
         static gboolean do_version;
@@ -1404,6 +1562,9 @@ main (int argc, char *argv[])
                 }
         }
 
+        /* register types that the builder might need */
+        type = um_strength_bar_get_type ();
+
         d->builder = gtk_builder_new ();
         d->um = um_user_manager_ref_default ();
 
