@@ -22,10 +22,20 @@
 #ifndef __UM_UTILS_H__
 #define __UM_UTILS_H__
 
+#include <gtk/gtk.h>
+
 G_BEGIN_DECLS
 
-gchar *um_compute_short_name (const gchar *name);
+void     setup_tooltip_with_embedded_icon (GtkWidget   *widget,
+                                           const gchar *text,
+                                           const gchar *placeholder,
+                                           const gchar *stock_id);
+gboolean show_tooltip_now                 (GtkWidget   *widget,
+                                           GdkEvent    *event);
 
+void     set_entry_validation_error       (GtkEntry    *entry,
+                                           const gchar *text);
+void     clear_entry_validation_error     (GtkEntry    *entry);
 G_END_DECLS
 
 #endif
